#!/bin/bash

test -f /etc/copy_videofiles.conf && . /etc/copy_videofiles.conf

test -n "$SVDRSend" || SVDRSend=/usr/lib/vdr/svdrpsend.pl # Fr Test ob Aufnahme luft
test -n "$Debug"    || Debug=""                           # Debug Ausgabe verbergen mit Debug="> /dev/null" / anzeigen mit Debug=""
test -n "$AllFiles" || AllFiles=1                         # Bei AllFiles=1 werden auch index.vdr ... verschoben bei AllFiles=0 nur die 001.vdr bis 999.vdr
test -n "$AllDirs"  || AllDirs=0                          # Bei AllDirs=1 werden auch die Verzeichnisse ohne Daten zur Prfung des erforderlichen Platzes mit eingetragen (trifft erst nach den Verzeichnissen mit Daten zu)
test -n "$Reserve"  || Reserve=20480                      # Platzreserve bei Berechnung Zielverzeichnis
test -n "$ErrorLog" || ErrorLog=/cp_err.log               # Aufzeichnung der Fehler
test 0 -eq $Reserve 2>/dev/null
if [ $? -ne 0 ]; then
  Reserve=20480
fi

#todo:
#  verzeichnisse aufnehmen ohne daten
#  find /video*/ -type d | awk '{ if ($0 !~ /\/video0\/.*/) { sub(/\/video.\//, ""); if (length($0)) print } }' | more
#  bei sub(/\/video.\   auch video02 erlauben
#  links erstellen geht nur mit video0 als Hauptverzeichnis

LinksUndStatistik ()
{
  # Zuerst geloeschte Links wieder herstellen
  for File in `find /video*/ -type f`
  do
    Verzeichnis=`echo $File | awk '{ split($0, p, "/"); printf "%s", p[2] }'`
    if [ "$Verzeichnis" != "video0" ]; then
      Aufnahme=`echo $File | awk '{ Anzahl = split($0, p, "/"); for (i=3; i < Anzahl; i++) printf "/%s", p[i] }'`
      Filename=`echo $File | awk '{ Anzahl = split($0, p, "/"); printf "%s", p[Anzahl] }'`
      mkdir -p "/video0$Aufnahme" 2>/dev/null
      ln -s "$File" "/video0$Aufnahme/$Filename" 2>/dev/null
    fi
  done
  # Ausgabe der Dateianzahl
  if [ $DateienAnzahl -gt 0 ]; then
    echo "insgesamt $DateienAnzahl Dateien mit $DateienGroesse kB verschoben"
  fi
  if [ $FehlerDateien -eq 0 -a $FehlerPlatz -eq 0 ]; then
    echo "Verarbeitung ohne Fehler beendet"
  fi
  if [ $FehlerDateien -gt 0 ]; then
    echo "Es sind $FehlerDateien Fehler bei der Verschiebung der Dateien aufgetreten"
  fi
  if [ $FehlerPlatz -gt 0 ]; then
    echo "Es wurden $FehlerPlatz Aufnahmen nicht bearbeitet, weil nicht ausreichen Platz vorhanden war"
  fi
}

CheckNext ()
{
# Paramter $1 = Hinweis bei Scriptstart, wann nchste Aufnahme beginnt
# Test ob eine Aufnahme luft / gleich beginnt (dann Abbruch des Scriptes)

  local Next
  Next=`$SVDRSend Next rel | awk '/^250/ { printf "%d", $3 }'`
  if [ $Next -lt 0 ]
  then
    Next=`expr 0 - $Next`
    echo "Abbruch: Aufnahme luft seit $Next Sekunden ..." >/dev/stderr
    if [ $# -eq 0 ]; then
      LinksUndStatistik
    fi
    exit 2
  fi

  if [ $Next -lt 300 ]
  then
    echo "Abbruch: Aufnahme luft in $Next Sekunden los ..." >/dev/stderr
    if [ $# -eq 0 ]; then
      LinksUndStatistik
    fi
    exit 1
  fi
  if [ $# -gt 0 ]; then
    echo "Nchste Aufnahme in $Next Sekunden."
  fi
}

AufnahmenUmbenennen ()
{
  local Link
  local LinkZiel
  local LinkName
  local LinkZielName
  local LinkZielVolu
  for Link in `find /video*/ -type l 2>/dev/null`
  do
    LinkZiel=`readlink $Link 2>/dev/null`
    if [ -f $LinkZiel ]; then
      LinkName=`echo $Link | awk '{ Anzahl = split($0, p, "/"); for (i=3; i < Anzahl; i++) printf "/%s", p[i] }'`
      LinkZielName=`echo $LinkZiel | awk '{ Anzahl = split($0, p, "/"); for (i=3; i < Anzahl; i++) printf "/%s", p[i] }'`
      LinkZielVolu=`echo $LinkZiel | awk '{ split($0, p, "/"); printf "%s", p[2] }'`
      if [ $? -eq 0 ]; then
        if [ "$LinkName" != "$LinkZielName" ]; then
	  echo "Umbenennung $LinkZiel nach $LinkName" $Debug
          mkdir -p /$LinkZielVolu$LinkName 2>/dev/null
	  mv $LinkZiel /$LinkZielVolu$LinkName/ 2>/dev/null
        fi
      fi
    fi
  done
}

LoescheLinksUndLeereVerzeichnisse ()
{
  find /video*/ -type l -exec rm {} \;
  while true
  do
    local Loeschung=0
    local Dir
    for Dir in `find /video*/ -type d -empty 2>/dev/null`
    do
      rmdir "$Dir" 2>/dev/null
      if [ $? -eq 0 ]; then
        let "Loeschung++"
      fi
    done
    if [ $Loeschung -eq 0 ]; then
      break
    fi
  done
}

Sonderfall ()
{
# Parameter $1 = Aufnahmename (Verzeichnis ohne /video?/)
# in dieser Funktion kann fr bestimmte Ordnernamen ein zugehriges video?-Verzeichnis festgelegt werden
# dieses wird dann als erstes auf freien Platz geprft

  local SpezialDir=""
  local i
  local f=0
  case `echo $1 | awk '{ split($0, p, "/"); printf "%s", tolower(p[1]) }'` in
    xfilme)        SpezialDir="video4";;
    xserien)       SpezialDir="video3";;
    xwissenschaft) SpezialDir="video2";;
  esac
  if [ "$SpezialDir" ]; then
    for (( i = 1; i <= ${#AVolume[@]}; i++ ));
    do
      if [ "${AVolume[$i]}" = "$SpezialDir" ]; then
        ASonder[$i]=2
        f=1
        break
      fi
    done
    if [ $f -eq 0 ]; then
      let "Count++"
      ASonder[$Count]=2
      AVolume[$Count]=$SpezialDir
      ASize[$Count]=0
    fi
  else
    SpezialDir="notuse"
  fi
  echo "Test Sonderfall $SpezialDir [$1]" $Debug
}

BestimmeFreieGroesse ()
{
  # Fr alle verwendeten Verzeichnisse wird der freie Platz bestimmt
  for (( i = 1; i <= ${#AVolume[@]}; i++ ));
  do
    AFree[$i]=`df /${AVolume[$i]}/ -P 2>/dev/null | tail -1 | awk '{ free = $4 }; END { printf "%d", free }'`
    if [ ${AFree[$i]} -eq 0 ]; then
      echo "df hat fr ${AVolume[$i]} keinen Wert geliefert" $Debug
    fi
  done
}

TauscheWerte ()
{
# Parameter $1 = 1. Tauschpartner
#           $2 = 2. Tauschpartner
# Hilfsfunktion fr Sortierung AFree wird erst nach Sortierung gefllt

  local TSonder=${ASonder[$1]}
  ASonder[$1]=${ASonder[$2]}
  ASonder[$2]=$TSonder

  local TVolume=${AVolume[$1]}
  AVolume[$1]=${AVolume[$2]}
  AVolume[$2]=$TVolume

  local TSize=${ASize[$1]}
  ASize[$1]=${ASize[$2]}
  ASize[$2]=$TSize

  return
}  

Sortierung ()
{
# Sortierung der Verzeichnisse nach verwendeter Gre unter Beachtung der Sonderzuweiseung (siehe Sonderfall)

  let "Comparisons = $Count - 1"
  Pass=1
  while [ $Comparisons -gt 0 ] # Beginning of outer loop (Bubble Sort from http://www.signaltonoise.net/library/Adv-Bash-Scr-HOWTO/arrays.html)
  do
    Index=1 # Reset index to start of array after each pass.
    while [ $Index -lt $Count ] # Beginning of inner loop
    do
      let "Next = $Index + 1"
      Tausch=0
      if [ ${ASonder[$Index]} -lt ${ASonder[$Next]} ]; then
        Tausch=1
      elif [ ${ASonder[$Index]} -eq ${ASonder[$Next]} ]; then
        if [ ${ASize[$Index]} -lt ${ASize[$Next]} ]; then
          Tausch=1
        fi
      fi
      if [ $Tausch -eq 1 ]; then
        # If out of order...
        TauscheWerte $Index $Next
      fi  
      let "Index++"
    done # End of inner loop
    let "Comparisons -= 1"
  done  # End of outer loop
}

Verschiebe ()
{
# Parameter $1 = Aufnahmename (Verzeichnis ohne /video?/)
#           $2 = Zielverzeichnisnummer in AVolume[]
# in dieser Funktion werden alle Daten aus den Verzeichnissen <> $2 in das mit der Nummer $2 verschoben

  local i
  local Files
  local File
  local Target="/${AVolume[$2]}/$1"
  local Error=0
  local ErrorText=""

  if [ ! -d "$Target" ]; then
    mkdir -p "$Target" >/dev/null 2>/dev/null
  fi
  if [ ! -d "$Target" ]; then
    echo "Stop: Verzeichnis $Target lst sich nicht anlegen" | tee -a $ErrorLog >/dev/stderr
    let "FehlerDateien++"
    return
  fi
  for (( i = 1; i <= $Count; i++ ));
  do
    if [ $2 -ne $i ]; then
      if [ $AllFiles -ne 0 ]; then
        Files=`ls "/${AVolume[$i]}/$1/" -A -1 2>/dev/null`
      else
        Files=`ls "/${AVolume[$i]}/$1/" -A -1 2>/dev/null | awk '{ if ($0 ~ /[0-9][0-9][0-9]\.vdr/) printf("%s\n", $0]) }'`
      fi  
      for File in $Files
      do
        if [ $File ]; then
          echo "Datei: $File von Verzeichnis: ${AVolume[$i]}" $debug
          if [ -e "$Target/$File" ]; then
            echo "Stop: Datei $File existiert bereits in $Target" | tee -a $ErrorLog >/dev/stderr
            let "FehlerDateien++"
            return
          fi
          echo "mv /${AVolume[$i]}/$1/$File $Target/$File" $debug
          ErrorText=`mv "/${AVolume[$i]}/$1/$File" "$Target/$File" 2>&1`
          Error=$?
          if [ ! $Error -eq 0 ]; then
            echo "Stop: Fehlercode $Error [$ErrorText] beim Verschieben der Datei $File Quelle: /${AVolume[$i]}/$1 Ziel: $Target" | tee -a $ErrorLog >/dev/stderr
            let "FehlerDateien++"
            return
          fi
          if [ ! -e "$Target/$File" ]; then
            echo "Stop: Datei $File ist nicht angekommen Ziel: $Target" | tee -a $ErrorLog >/dev/stderr
            let "FehlerDateien++"
            return
          fi
          if [ -e "/${AVolume[$i]}/$1/$File" ]; then
            echo "Stop: Datei $File ist noch im Quellverzeichnis vorhanden Quelle: /${AVolume[$i]}/$1" | tee -a $ErrorLog >/dev/stderr
            let "FehlerDateien++"
            return
          fi
          let "DateienAnzahl++"
        fi
      done
      let "DateienGroesse += ${ASize[$i]}"
    fi
  done # for i
}

BearbeiteAufnahme ()
{
# Parameter $1 = Aufnahmename (Verzeichnis ohne /video?/)
# Suchen der Dateien fr eine Aufnahme, Sortierung und Verschiebung der Daten

  CheckNext
  Count=0
  unset ASonder
  unset AVolume
  unset ASize
  unset AFree
  # Suchen aller Verwendeten Aufnahmeverzeichnissen
  for Dir in `find /video*/$1 -type d`
  do
    if [ $AllFiles -ne 0 ]; then
      Size=`ls $Dir -lA 2>/dev/null | awk '{ total += $5 }; END { printf "%d\n", total / 1024 }'`
    else
      Size=`ls $Dir -lA 2>/dev/null | awk '{ if ($9 ~ /[0-9][0-9][0-9]\.vdr/) total += $5 }; END { printf "%d\n", total / 1024 }'`
    fi
    Anzahl=0
    if [ $Size -eq 0 -a $AllFiles -ne 0 ]; then
      Anzahl=`ls $Dir -1 2>/dev/null | wc -l`
    fi
    if [ $Size -gt 0 -o $Anzahl -gt 0 ]; then
      # Speichern der Werte fr sptere Auswertung in 4 Arrays (Sonderfall, Video-Verzeichnis, verwendete und freie Gre)
      let "Count++"
      #Count=`expr $Count + 1`
      ASonder[$Count]=1
      AVolume[$Count]=`echo $Dir | awk '{ split($0, p, "/"); printf "%s", p[2] }'`
      ASize[$Count]=$Size
    fi  
  done
  # Test auf Spezialverzeichniszuweisung
  Sonderfall $1
  # Sortierung nach belegter Gre
  Sortierung
  # Aktuellen freien Platz holen
  BestimmeFreieGroesse
  local i
  if [ ${#AVolume[@]} -gt 1 ]; then
    # es ist mehr als 1 Verzeichnis vorhanden
    echo "Aufnahme: $1" $Debug
    # Ausgabe der gefundenen und sortierten Werte
    for (( i = 1; i <= $Count; i++ ));
    do
      echo "${AVolume[$i]} ${ASize[$i]} ${AFree[$i]}" | awk '{ printf "%-10s Gre =%10d kB Frei =%10d kB\n", $1, $2, $3 }' $Debug
    done
    # Suche nach einem Verzeichnis mit gengend Platz fr alle Dateien
    for (( i = 1; i <= $Count; i++ ));
    do
      local SizeCopy=$Reserve # (20MB?) Reserve fr Berechnung Platzbedarf
      for (( j = 1; j <= $Count; j++ ));
      do
        if [ $i -ne $j ]; then
          let "SizeCopy += ${ASize[$j]}"
        fi
      done
      if [ $SizeCopy -lt ${AFree[$i]} ]; then 
        # Verzeichnis gefunden, alle Dateien dorthin verschieben
	let "SizeCopy -= $Reserve" # Reserve abziehen
        echo "$SizeCopy ${AVolume[$i]}" | awk '{ printf "Verschiebe alles [%10d kB ] nach %s\n", $1, $2 }' $Debug
        Verschiebe "$1" $i
        break
      fi # SizeCopy < Frei
      if [ $i -eq $Count ]; then
        echo "Stop: Kein Zielverzeichnis mit ausreichend Platz fr alle Dateien gefunden Aufnahme: $1" >/dev/stderr
        let "FehlerPlatz++"
      fi
    done # for i
  fi  
}

# Begin Hauptprogramm

if [ "$1" == "h" ]; then
  echo "opt l --> Aufnahmen umbenennen und Links lschen"
  echo "opt n --> Aufnahmen umbenennen, Links lschen und wieder erstellen"
  echo "          ohne die Daten zusammenzufassen"
  echo "opt w --> Aufnahmen umbenennen, Links lschen und Daten zusammenfassen"
  exit 0
fi

DateienAnzahl=0  # insgesamt verschobene Dateien
DateienGroesse=0 # insgesamt verschobene Gre (kb)
FehlerDateien=0  # bei der Verschiebung sind Fehler aufgetreten
FehlerPlatz=0    # bei Prfen auf freien Platz war keiner mehr da

CheckNext Hinweis
echo "Aufnahmen werden umbenant ..."
AufnahmenUmbenennen
echo "Links und leere Verzeichnisse werden entfernt ... (1)"
LoescheLinksUndLeereVerzeichnisse
if [ "$1" == "l" ]; then
  exit 0
fi
if [ "$1" != "n" ]; then
  echo "Dateizusammenfassung ..."
  if [ $AllDirs -ne 0 ]; then
    # Alle Aufnahmen werden geprft
    Records=`find /video*/ -type d -name *.rec | awk '{ sub(/\/video.\//, ""); print $1 }' | sort | awk '{ if (line2 != $0) { line2 = $0; print } }'`
  else
    # Nur Aufnahmen mit 2 oder mehr Verzeichnissen werden geprft
    Records=`find /video*/ -type d -name *.rec | awk '{ sub(/\/video.\//, ""); print $1 }' | sort | awk '{ if (line1 != $0) line1 = $0; else if (line2 != $0) { line2 = $0; print } }'`
  fi
  for Record in $Records
  do
    BearbeiteAufnahme $Record
  done # for Record

  echo "Links und leere Verzeichnisse werden entfernt ... (2)"
  LoescheLinksUndLeereVerzeichnisse
fi
if [ "$1" == "w" ]; then
  exit 0
fi
echo "Erstelle Links ..."
LinksUndStatistik
